package org.openxma.dsl.generator.helper;

import org.openxma.dsl.core.CoreFactory;
import org.openxma.dsl.core.model.AdditiveExpr;
import org.openxma.dsl.core.model.AtomicBoolExpr;
import org.openxma.dsl.core.model.CondANDExpr;
import org.openxma.dsl.core.model.CondANDRights;
import org.openxma.dsl.core.model.CondORExpr;
import org.openxma.dsl.core.model.CondORRights;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.MultiplicativeExpr;
import org.openxma.dsl.core.model.ParenExpr;
import org.openxma.dsl.core.model.RelationalExpr;

public class XmadslExpression {
	
	/**
	 * Join 2 expressions by the operator 'and'
	 * 
	 * @param exp1
	 * @param exp2
	 * @return
	 */
	public static EqualityExpr and(EqualityExpr exp1, EqualityExpr exp2) {
		
		EqualityExpr joinedExpression = CoreFactory.eINSTANCE.createEqualityExpr();
		
		CondORExpr orExpr = CoreFactory.eINSTANCE.createCondORExpr();
		CondANDExpr andExpr = CoreFactory.eINSTANCE.createCondANDExpr();		
		joinedExpression.setLeft(orExpr);					
		orExpr.setLeft(andExpr);
		
		// Left
		AtomicBoolExpr atomicBoolExprLeft = createAtomicBoolExpr(exp1);					
		andExpr.setLeft(atomicBoolExprLeft);					
		
		// Right
		CondANDRights condAndRightsExpr = CoreFactory.eINSTANCE.createCondANDRights();
		AtomicBoolExpr atomicBoolExprRight = createAtomicBoolExpr(exp2);			
		andExpr.getRights().add(condAndRightsExpr);														
		condAndRightsExpr.setRight(atomicBoolExprRight);
		
		return joinedExpression;
	}
	
	/**
	 * Join 2 expressions by the operator 'or'
	 * 
	 * @param exp1
	 * @param exp2
	 * @return
	 */
	public static EqualityExpr or(EqualityExpr exp1, EqualityExpr exp2) {
		
		EqualityExpr joinedExpression = CoreFactory.eINSTANCE.createEqualityExpr();		
		CondORExpr orExpr = CoreFactory.eINSTANCE.createCondORExpr();	
		joinedExpression.setLeft(orExpr);					
		
		// Left
		CondANDExpr andLeftExpr = CoreFactory.eINSTANCE.createCondANDExpr();			
		AtomicBoolExpr atomicBoolExprLeft = createAtomicBoolExpr(exp1);
		orExpr.setLeft(andLeftExpr);		
		andLeftExpr.setLeft(atomicBoolExprLeft);					
		
		// Right
		CondORRights condORRightsExpr = CoreFactory.eINSTANCE.createCondORRights();
		CondANDExpr andRightExpr = CoreFactory.eINSTANCE.createCondANDExpr();		
		AtomicBoolExpr atomicBoolExprRight = createAtomicBoolExpr(exp2);
		orExpr.getRights().add(condORRightsExpr);														
		condORRightsExpr.setRight(andRightExpr);
		andRightExpr.setLeft(atomicBoolExprRight);			
		
		return joinedExpression;
	}	

	/**
	 * Wrap a EqualityExpr in a AtomicBoolExpr
	 * 
	 * @param equalityExpr
	 * @return
	 */
	public static AtomicBoolExpr createAtomicBoolExpr(EqualityExpr equalityExpr) {
		
		RelationalExpr relExpr = CoreFactory.eINSTANCE.createRelationalExpr();
		
		AdditiveExpr addExpr = CoreFactory.eINSTANCE.createAdditiveExpr();
		MultiplicativeExpr multExpr = CoreFactory.eINSTANCE.createMultiplicativeExpr();
		ParenExpr parExpr = CoreFactory.eINSTANCE.createParenExpr();
		
		relExpr.setLeft(addExpr);	
		addExpr.setLeft(multExpr);
		multExpr.setLeft(parExpr);					
		parExpr.setExpr(equalityExpr);
		
		return relExpr;
	}	

}
